'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
var doTextMatchPattern = exports.doTextMatchPattern = function doTextMatchPattern(text, pattern) {
  var containsPattern = pattern.includes('*') || pattern.includes('#');

  if (!containsPattern) {
    return text === pattern;
  }

  var regExp = pattern.replace(/\./g, '\\.');
  regExp = regExp.replace(/\*/g, '\\w+');
  regExp = regExp.replace(/#/g, '.+?');
  regExp = '^' + regExp + '$';

  regExp = new RegExp(regExp);

  return regExp.test(text);
};

exports.default = function () {
  return {
    subscribers: {},
    subscribe: function subscribe() {
      var _this = this;

      // if we don't provide 3 arguments then we get 1st argument as topic and 2nd
      //  as callback, channel is 'default'

      var channel = arguments[0];
      var topicPattern = arguments[1];
      var callback = arguments[2];

      if (arguments.length === 2) {
        channel = 'default';
        topicPattern = arguments[0];
        callback = arguments[1];
      }

      if (this.subscribers[channel] === undefined) {
        this.subscribers[channel] = {};
      }

      if (this.subscribers[channel][topicPattern] === undefined) {
        this.subscribers[channel][topicPattern] = { subscribers: [], queued: [] };
      }

      this.subscribers[channel][topicPattern].subscribers.push(callback);

      var _iteratorNormalCompletion = true;
      var _didIteratorError = false;
      var _iteratorError = undefined;

      try {
        for (var _iterator = this.subscribers[channel][topicPattern].queued[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
          var queued = _step.value;

          callback(queued.data, {
            channel: queued.channel,
            topic: queued.topic
          });
        }
      } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion && _iterator.return) {
            _iterator.return();
          }
        } finally {
          if (_didIteratorError) {
            throw _iteratorError;
          }
        }
      }

      this.subscribers[channel][topicPattern].queued = [];

      return {
        unsubscribe: function unsubscribe() {
          var index = _this.subscribers[channel][topicPattern].subscribers.findIndex(function (s) {
            return s === callback;
          });

          if (index !== -1) {
            _this.subscribers[channel][topicPattern].subscribers.splice(index, 1);
          }
        }
      };
    },
    publish: function publish() {
      var channel = arguments[0];
      var topic = arguments[1];
      var data = arguments[2];
      var onlyOnce = arguments[3];

      if (arguments.length === 3) {
        channel = arguments[0];
        topic = arguments[1];
        data = arguments[2];
        onlyOnce = false;
      } else if (arguments.length === 2) {
        channel = 'default';
        topic = arguments[0];
        data = arguments[1];
        onlyOnce = false;
      }

      if (this.subscribers[channel] === undefined || this.subscribers[channel][topic] === undefined || this.subscribers[channel][topic].subscribers.length == 0) {
        this.subscribers[channel] = {
          topic: {
            subscribers: [],
            queued: [{
              channel: channel,
              topic: topic,
              data: data
            }]
          }
        };
        return;
      }

      var matchedTopicPatterns = Object.keys(this.subscribers[channel]).filter(function (t) {
        return doTextMatchPattern(topic, t);
      });

      var _iteratorNormalCompletion2 = true;
      var _didIteratorError2 = false;
      var _iteratorError2 = undefined;

      try {
        for (var _iterator2 = matchedTopicPatterns[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
          var topicPattern = _step2.value;

          if (!onlyOnce) {
            this.subscribers[channel][topicPattern].subscribers.forEach(function (s) {
              return s(data, {
                channel: channel,
                topic: topic
              });
            });
          } else {
            var subCount = this.subscribers[channel][topicPattern].subscribers.length;
            this.subscribers[channel][topicPattern].subscribers[subCount - 1](data, {
              channel: channel,
              topic: topic
            });
          }
        }
      } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
      } finally {
        try {
          if (!_iteratorNormalCompletion2 && _iterator2.return) {
            _iterator2.return();
          }
        } finally {
          if (_didIteratorError2) {
            throw _iteratorError2;
          }
        }
      }
    }
  };
};
